@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,8 @@ import urllib
 import urllib2
 
 from . import urlbase, get_index_html, get_url_from
-from .. import log, LOG_CHECK, fileutil, firefox
+from .. import log, LOG_CHECK, fileutil, LinkCheckerError
+from ..bookmarks import firefox
 from .const import WARN_FILE_MISSING_SLASH, WARN_FILE_SYSTEM_PATH
 
 
@@ -53,9 +54,7 @@ def prepare_urlpath_for_nt (path):
 
 
 def get_nt_filename (path):
-    """
-    Return case sensitive filename for NT path.
-    """
+    """Return case sensitive filename for NT path."""
     head, tail = os.path.split(path)
     if not tail:
         return path
@@ -66,6 +65,17 @@ def get_nt_filename (path):
     return path
 
 
+def get_os_filename (path):
+    """Return filesystem path for given URL path."""
+    if os.name == 'nt':
+        path = prepare_urlpath_for_nt(path)
+    res = urllib.url2pathname(fileutil.pathencode(path))
+    if os.name == 'nt' and res.endswith(':') and len(res) == 2:
+        # Work around http://bugs.python.org/issue11474
+        res += os.sep
+    return res
+
+
 def is_absolute_path (path):
     """Check if given path is absolute. On Windows absolute paths start
     with a drive letter. On all other systems absolute paths start with
@@ -113,6 +123,18 @@ class FileUrl (urlbase.UrlBase):
         """
         Calls super.build_url() and adds a trailing slash to directories.
         """
+        if self.parent_url is not None:
+            # URL joining with the parent URL only works if the query
+            # of the base URL are removed first.
+            # Otherwise the join function thinks the query is part of
+            # the file name.
+            from .urlbase import url_norm
+            # norm base url - can raise UnicodeError from url.idna_encode()
+            base_url, is_idn = url_norm(self.base_url, self.encoding)
+            urlparts = list(urlparse.urlsplit(base_url))
+            # ignore query part for filesystem urls
+            urlparts[3] = ''
+            self.base_url = urlparse.urlunsplit(urlparts)
         super(FileUrl, self).build_url()
         # ignore query and fragment url parts for filesystem urls
         self.urlparts[3] = self.urlparts[4] = ''
@@ -164,6 +186,8 @@ class FileUrl (urlbase.UrlBase):
     def read_content (self):
         """Return file content, or in case of directories a dummy HTML file
         with links to the files."""
+        if self.size > self.MaxFilesizeBytes:
+            raise LinkCheckerError(_("File size too large"))
         if self.is_directory():
             data = get_index_html(get_files(self.get_os_filename()))
             if isinstance(data, unicode):
@@ -199,10 +223,7 @@ class FileUrl (urlbase.UrlBase):
         @return: file name
         @rtype: string
         """
-        path = self.urlparts[2]
-        if os.name == 'nt':
-            path = prepare_urlpath_for_nt(path)
-        return fileutil.pathencode(urllib.url2pathname(path))
+        return get_os_filename(self.urlparts[2])
 
     def get_temp_filename (self):
         """Get filename for content to parse."""
@@ -233,9 +254,7 @@ class FileUrl (urlbase.UrlBase):
             return self.get_content_type() in self.ContentMimetypes
 
     def parse_url (self):
-        """
-        Parse file contents for new links to check.
-        """
+        """Parse file contents for new links to check."""
         if self.is_directory():
             self.parse_html()
         elif firefox.has_sqlite and firefox.extension.search(self.url):
@@ -255,6 +274,8 @@ class FileUrl (urlbase.UrlBase):
             self.aggregate.urlqueue.put(url_data)
 
     def get_content_type (self):
+        """Return URL content type, or an empty string if content
+        type could not be found."""
         if self.content_type is None:
             if self.url:
                 self.content_type = fileutil.guess_mimetype(self.url, read=self.get_content)
@@ -263,8 +284,7 @@ class FileUrl (urlbase.UrlBase):
         return self.content_type
 
     def get_intern_pattern (self):
-        """
-        Get pattern for intern URL matching.
+        """Get pattern for intern URL matching.
 
         @return non-empty regex pattern or None
         @rtype String or None
