@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@ from . import proxysupport, httpurl, internpaturl, get_index_html
 from .const import WARN_FTP_MISSING_SLASH
 
 DEFAULT_TIMEOUT_SECS = 300
-MAX_FTP_FILESIZE = 1024*1024*10 # 10MB
 
 
 class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
@@ -195,7 +194,9 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
 
     def is_directory (self):
         """See if URL target is a directory."""
-        return self.url.endswith('/')
+        # either the path is empty, or ends with a slash
+        path = self.urlparts[2]
+        return (not path) or path.endswith('/')
 
     def parse_url (self):
         """Parse URL target for links."""
@@ -206,6 +207,8 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         getattr(self, "parse_"+key)()
 
     def get_content_type (self, read=None):
+        """Return URL content type, or an empty string if content
+        type could not be found."""
         if self.content_type is None:
             self.content_type = fileutil.guess_mimetype(self.url, read=read)
         return self.content_type
@@ -224,7 +227,7 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             def stor_data (s):
                 """Helper method storing given data"""
                 # limit the download size
-                if (buf.tell() + len(s)) > MAX_FTP_FILESIZE:
+                if (buf.tell() + len(s)) > self.MaxFilesizeBytes:
                     raise LinkCheckerError(_("FTP file size too large"))
                 buf.write(s)
             self.url_connection.retrbinary(ftpcmd, stor_data)
