@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,9 @@ import errno
 import zlib
 import socket
 from cStringIO import StringIO
-import Cookie
 
 from .. import (log, LOG_CHECK, gzip2 as gzip, strformat, url as urlutil,
-    httplib2 as httplib, LinkCheckerError, configuration)
+    httplib2 as httplib, LinkCheckerError, get_link_pat, httputil)
 from . import (internpaturl, proxysupport, httpheaders as headers, urlbase,
     get_url_from)
 # import warnings
@@ -254,10 +253,9 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                        tag=WARN_HTTP_AUTH_UNKNOWN)
                     return
                 if not self.auth:
-                    import base64
                     _user, _password = self.get_user_password()
                     self.auth = "Basic " + \
-                        base64.encodestring("%s:%s" % (_user, _password))
+                        httputil.encode_base64("%s:%s" % (_user, _password))
                     log.debug(LOG_CHECK,
                         "Authentication %s/%s", _user, _password)
                     continue
@@ -272,9 +270,11 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 mime = self.get_content_type()
                 poweredby = self.headers.get('X-Powered-By', '')
                 server = self.headers.get('Server', '')
-                if mime in ('application/octet-stream', 'text/plain') and \
-                  (poweredby.startswith('Zope') or server.startswith('Zope')):
-                    # Zope server could not get Content-Type with HEAD
+                if ((mime in ('application/octet-stream', 'text/plain') and
+                  (poweredby.startswith('Zope') or server.startswith('Zope')))
+                 or ('ASP.NET' in poweredby and 'Microsoft-IIS' in server)):
+                    # Zope or IIS server could not get Content-Type with HEAD
+                    # see http://intermapper.com.dev4.silvertech.net/bogus.aspx
                     self.method = "GET"
                     self.aliases = []
                     self.fallback_get = True
@@ -286,96 +286,151 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         """Return content MIME type or empty string."""
         if self.content_type is None:
             if self.headers:
-               self.content_type = headers.get_content_type(self.headers)
+                self.content_type = headers.get_content_type(self.headers)
             else:
                 self.content_type = u""
         return self.content_type
 
     def follow_redirections (self, response, set_result=True):
-        """
-        Follow all redirections of http response.
-        """
+        """Follow all redirections of http response."""
         log.debug(LOG_CHECK, "follow all redirections")
         redirected = self.url
         tries = 0
         while response.status in [301, 302] and self.headers and \
               tries < self.max_redirects:
-            newurl = self.headers.getheader("Location",
-                         self.headers.getheader("Uri", ""))
-            # make new url absolute and unicode
-            newurl = unicode_safe(newurl)
-            newurl = urlparse.urljoin(redirected, newurl)
-            log.debug(LOG_CHECK, "Redirected to %r", newurl)
-            self.add_info(_("Redirected to `%(url)s'.") % {'url': newurl})
-            # norm base url - can raise UnicodeError from url.idna_encode()
-            redirected, is_idn = urlbase.url_norm(newurl)
-            log.debug(LOG_CHECK, "Norm redirected to %r", redirected)
-            urlparts = strformat.url_unicode_split(redirected)
-            if urlparts[1] != self.urlparts[1]:
-                # check extern filter again
-                self.set_extern(redirected)
-                if self.extern[0] and self.extern[1]:
-                    if set_result:
-                        self.check301status(response)
-                        self.add_info(
-                             _("The redirected URL is outside of the domain "
-                               "filter, checked only syntax."))
-                        self.set_result(u"filtered")
-                    return -1, response
-            # check robots.txt allowance again
-            if not self.allows_robots(redirected):
-                if set_result:
-                    self.add_warning(
-                       _("Access to redirected URL denied by robots.txt, "
-                         "checked only syntax."),
-                       tag=WARN_HTTP_ROBOTS_DENIED)
-                    self.set_result(u"syntax OK")
-                return -1, response
-            # see about recursive redirect
-            all_seen = [self.cache_url_key] + self.aliases
-            if redirected in all_seen:
-                if self.method == "HEAD" and self.method_get_allowed:
-                    # Microsoft servers tend to recurse HEAD requests
-                    # fall back to the original url and use GET
-                    return self.max_redirects, response
-                recursion = all_seen + [redirected]
-                if set_result:
-                    self.set_result(
-                          _("recursive redirection encountered:\n %(urls)s") %
-                            {"urls": "\n  => ".join(recursion)}, valid=False)
-                return -1, response
-            if urlparts[0] == self.scheme or urlparts[0] in ('http', 'https'):
-                # remember redirected url as alias
-                self.aliases.append(redirected)
-            else:
-                # in case of changed scheme make new URL object
-                newobj = get_url_from(
-                          redirected, self.recursion_level, self.aggregate,
-                          parent_url=self.parent_url, base_ref=self.base_ref,
-                          line=self.line, column=self.column, name=self.name)
-                if set_result:
-                    self.add_warning(
-                     _("Redirection to URL `%(newurl)s' with different scheme"
-                       " found; the original URL was `%(url)s'.") %
-                     {"url": self.url, "newurl": newobj.url},
-                     tag=WARN_HTTP_WRONG_REDIRECT)
-                    self.set_result(u"syntax OK")
-                # append new object to queue
-                self.aggregate.urlqueue.put(newobj)
-                # pretend to be finished and logged
-                return -1, response
-            # note: urlparts has to be a list
-            self.urlparts = urlparts
+            num, response = self.follow_redirection(response, set_result, redirected)
+            if num == -1:
+                return num, response
+            redirected = urlparse.urlunsplit(self.urlparts)
+            tries += num
+        return tries, response
+
+    def follow_redirection (self, response, set_result, redirected):
+        """Follow one redirection of http response."""
+        newurl = self.headers.getheader("Location",
+                     self.headers.getheader("Uri", ""))
+        # make new url absolute and unicode
+        newurl = urlparse.urljoin(redirected, unicode_safe(newurl))
+        log.debug(LOG_CHECK, "Redirected to %r", newurl)
+        self.add_info(_("Redirected to `%(url)s'.") % {'url': newurl})
+        # norm base url - can raise UnicodeError from url.idna_encode()
+        redirected, is_idn = urlbase.url_norm(newurl)
+        log.debug(LOG_CHECK, "Norm redirected to %r", redirected)
+        urlparts = strformat.url_unicode_split(redirected)
+        if not self.check_redirection_scheme(redirected, urlparts, set_result):
+            return -1, response
+        if not self.check_redirection_domain(redirected, urlparts,
+                                             set_result, response):
+            return -1, response
+        if not self.check_redirection_robots(redirected, set_result):
+            return -1, response
+        num = self.check_redirection_recursion(redirected, set_result)
+        if num != 0:
+            return num, response
+        if not self.check_redirection_newscheme (redirected, urlparts, set_result):
+            return -1, response
+        # remember redirected url as alias
+        self.aliases.append(redirected)
+        # note: urlparts has to be a list
+        self.urlparts = urlparts
+        if set_result:
+            self.check301status(response)
+        # check cache again on the changed URL
+        if self.aggregate.urlqueue.checked_redirect(redirected, self):
+            return -1, response
+        # new response data
+        response.close()
+        response = self._try_http_response()
+        return 1, response
+
+    def check_redirection_scheme (self, redirected, urlparts, set_result):
+        """Return True if redirection scheme is ok, else False."""
+        if urlparts[0] in ('ftp', 'http', 'https'):
+            return True
+        # in case of changed scheme make new URL object
+        # For security reasons do not allow redirects to protocols
+        # other than HTTP, HTTPS or FTP.
+        if set_result:
+            self.add_warning(
+              _("Redirection to url `%(newurl)s' is not allowed.") %
+              {'newurl': redirected})
+            self.set_result(u"syntax OK")
+        return False
+
+    def check_redirection_domain (self, redirected, urlparts, set_result, response):
+        """Return True if redirection domain is ok, else False."""
+        if urlparts[1] == self.urlparts[1]:
+            # URL domain did not change
+            return True
+        # URL domain changed
+        if self.recursion_level == 0 and urlparts[0] in ('http', 'https'):
+            # Add intern patterns for redirection of URLs given by the
+            # user for HTTP schemes.
+            pat = internpaturl.get_intern_pattern(redirected)
+            log.debug(LOG_CHECK, "Add intern pattern %r", pat)
+            self.aggregate.config['internlinks'].append(get_link_pat(pat))
+            return True
+        # check extern filter again
+        self.set_extern(redirected)
+        if self.extern[0] and self.extern[1]:
             if set_result:
                 self.check301status(response)
-            # check cache again on the changed URL
-            if self.aggregate.urlqueue.checked_redirect(redirected, self):
-                return -1, response
-            # new response data
-            response.close()
-            response = self._try_http_response()
-            tries += 1
-        return tries, response
+                self.add_info(_("The redirected URL is outside of the domain "
+                              "filter, checked only syntax."))
+                self.set_result(u"filtered")
+            return False
+        return True
+
+    def check_redirection_robots (self, redirected, set_result):
+        """Check robots.txt allowance for redirections. Return True if
+        allowed, else False."""
+        if self.allows_robots(redirected):
+            return True
+        if set_result:
+            self.add_warning(
+               _("Access to redirected URL denied by robots.txt, "
+                 "checked only syntax."), tag=WARN_HTTP_ROBOTS_DENIED)
+            self.set_result(u"syntax OK")
+        return False
+
+    def check_redirection_recursion (self, redirected, set_result):
+        """Check for recursive redirect. Return zero if no recursion
+        detected, max_redirects for recursion with HEAD request,
+        -1 otherwise."""
+        all_seen = [self.cache_url_key] + self.aliases
+        if redirected not in all_seen:
+            return 0
+        if self.method == "HEAD" and self.method_get_allowed:
+            # Microsoft servers tend to recurse HEAD requests
+            # fall back to the original url and use GET
+            return self.max_redirects
+        if set_result:
+            urls = "\n  => ".join(all_seen + [redirected])
+            self.set_result(_("recursive redirection encountered:\n %(urls)s") %
+                            {"urls": urls}, valid=False)
+        return -1
+
+    def check_redirection_newscheme (self, redirected, urlparts, set_result):
+        """Check for HTTP(S)/FTP redirection. Return True for HTTP(S)
+        redirection, False for FTP."""
+        if urlparts[0] in ('http', 'https'):
+            return True
+        # ftp scheme
+        assert urlparts[0] == 'ftp', 'Invalid redirection %r' % redirected
+        newobj = get_url_from(
+                  redirected, self.recursion_level, self.aggregate,
+                  parent_url=self.parent_url, base_ref=self.base_ref,
+                  line=self.line, column=self.column, name=self.name)
+        if set_result:
+            self.add_warning(
+             _("Redirection to URL `%(newurl)s' with different scheme"
+               " found; the original URL was `%(url)s'.") %
+             {"url": self.url, "newurl": newobj.url},
+             tag=WARN_HTTP_WRONG_REDIRECT)
+            self.set_result(u"syntax OK")
+        # append new object to queue
+        self.aggregate.urlqueue.put(newobj)
+        return False
 
     def check301status (self, response):
         """If response page has been permanently moved add a warning."""
@@ -409,17 +464,15 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             # store cookies for valid links
             if self.aggregate.config['storecookies']:
                 for c in self.cookies:
-                    self.add_info(_("Sent cookie: %(cookie)s.") %
-                                  {"cookie": c})
-                try:
-                    out = self.aggregate.cookies.add(self.headers,
-                                                     self.urlparts[0],
-                                                     self.urlparts[1],
-                                                     self.urlparts[2])
-                except Cookie.CookieError, msg:
-                    self.add_warning(_("Could not store cookies: %(msg)s.") %
-                                     {'msg': str(msg)},
-                                     tag=WARN_HTTP_COOKIE_STORE_ERROR)
+                    self.add_info(_("Sent Cookie: %(cookie)s.") %
+                                  {"cookie": c.client_header_value()})
+                errors = self.aggregate.cookies.add(self.headers,
+                    self.urlparts[0], self.urlparts[1], self.urlparts[2])
+                if errors:
+                    self.add_warning(
+                      _("Could not store cookies from headers: %(error)s.") %
+                       {'error': "\n".join(errors)},
+                       tag=WARN_HTTP_COOKIE_STORE_ERROR)
             if response.status >= 200:
                 self.set_result(u"%r %s" % (response.status, response.reason))
             else:
@@ -489,24 +542,17 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if (self.parent_url and
             self.parent_url.startswith(('http://', 'https://'))):
             self.url_connection.putheader("Referer", self.parent_url)
-        self.url_connection.putheader("User-Agent", configuration.UserAgent)
+        self.url_connection.putheader("User-Agent",
+            self.aggregate.config["useragent"])
         self.url_connection.putheader("Accept-Encoding",
                                   "gzip;q=1.0, deflate;q=0.9, identity;q=0.5")
         if self.aggregate.config['sendcookies']:
-            scheme = self.urlparts[0]
-            host = self.urlparts[1]
-            port = urlutil.default_ports.get(scheme, 80)
-            host, port = urllib.splitnport(host, port)
-            path = self.urlparts[2]
-            self.cookies = self.aggregate.cookies.get(scheme, host, port, path)
-            for c in self.cookies:
-                name = c.client_header_name()
-                value = c.client_header_value()
-                self.url_connection.putheader(name, value)
+            self.send_cookies()
         self.url_connection.endheaders()
         response = self.url_connection.getresponse(True)
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
+        self.content_type = None
         self.persistent = not response.will_close
         if self.persistent and self.method == "HEAD":
             # Some servers send page content after a HEAD request,
@@ -515,6 +561,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             # the connection after HEAD.
             # Example: http://www.empleo.gob.mx (Apache/1.3.33 (Unix) mod_jk)
             self.persistent = False
+        # Note that for POST method the connection should also be closed,
+        # but this method is never used.
         if self.persistent and (self.method == "GET" or
            self.headers.getheader("Content-Length") != "0"):
             # always read content from persistent connections
@@ -525,6 +573,37 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             response.reason = httplib.responses[response.status]
         return response
 
+    def send_cookies (self):
+        """Add cookie headers to request."""
+        scheme = self.urlparts[0]
+        host = self.urlparts[1]
+        port = urlutil.default_ports.get(scheme, 80)
+        host, port = urllib.splitnport(host, port)
+        path = self.urlparts[2]
+        self.cookies = self.aggregate.cookies.get(scheme, host, port, path)
+        if not self.cookies:
+            return
+        # add one cookie header with all cookie data
+        # this is limited by maximum header length
+        headername = "Cookie"
+        headervalue = ""
+        max_value_len = headers.MAX_HEADER_BYTES - len(headername) - 2
+        for c in self.cookies:
+            cookievalue = c.client_header_value()
+            if "version" in c.attributes:
+                # add separate header for explicit versioned cookie
+                if headervalue:
+                    self.url_connection.putheader(headername, headervalue)
+                self.url_connection.putheader(headername, cookievalue)
+                headervalue = ""
+                continue
+            if (len(headervalue) + len(cookievalue)) < max_value_len:
+                headervalue += cookievalue
+            else:
+                log.debug(LOG_CHECK, "Discard too-long cookie %r", cookievalue)
+        if headervalue:
+            self.url_connection.putheader(headername, headervalue)
+
     def get_http_object (self, host, scheme):
         """
         Open a HTTP connection.
@@ -568,10 +647,13 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         response = self._try_http_response()
         response = self.follow_redirections(response, set_result=False)[1]
         self.headers = response.msg
+        self.content_type = None
         # Re-read size info, since the GET request result could be different
         # than a former HEAD request.
         self.add_size_info()
         if self._data is None:
+            if self.size > self.MaxFilesizeBytes:
+                raise LinkCheckerError(_("File size too large"))
             self._read_content(response)
         data, size = self._data, self._size
         self._data = self._size = None
