@@ -39,7 +39,8 @@ from .const import (WARN_URL_EFFECTIVE_URL,
     WARN_URL_ERROR_GETTING_CONTENT, WARN_URL_OBFUSCATED_IP,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
     WARN_URL_CONTENT_SIZE_TOO_LARGE, WARN_URL_CONTENT_SIZE_ZERO,
-    WARN_URL_CONTENT_SIZE_UNEQUAL, ExcList, ExcSyntaxList, ExcNoCacheList)
+    WARN_URL_CONTENT_SIZE_UNEQUAL, WARN_URL_WHITESPACE,
+    ExcList, ExcSyntaxList, ExcNoCacheList)
 
 # helper alias
 unicode_safe = strformat.unicode_safe
@@ -86,8 +87,12 @@ class UrlBase (object):
         "application/msword": "word",
         "text/plain+linkchecker": "text",
         "text/plain+opera": "opera",
+        "text/plain+chromium": "chromium",
     }
 
+    # Set maximum file size for downloaded files in bytes.
+    MaxFilesizeBytes = 1024*1024*5
+
     def __init__ (self, base_url, recursion_level, aggregate,
                   parent_url=None, base_ref=None, line=-1, column=-1,
                   name=u"", url_encoding=None):
@@ -104,9 +109,9 @@ class UrlBase (object):
         @param name: name of url or empty
         @param url_encoding: encoding of URL or None
         """
+        self.reset()
         self.init(base_ref, base_url, parent_url, recursion_level,
                   aggregate, line, column, name, url_encoding)
-        self.reset()
         self.check_syntax()
 
     def init (self, base_ref, base_url, parent_url, recursion_level,
@@ -115,8 +120,10 @@ class UrlBase (object):
         Initialize internal data.
         """
         self.base_ref = base_ref
-        # note that self.base_url must not be modified
-        self.base_url = base_url
+        self.base_url = base_url.strip() if base_url else base_url
+        if self.base_url != base_url:
+            self.add_warning(_("Leading or trailing whitespace in URL `%(url)s'.") %
+                               {"url": base_url}, tag=WARN_URL_WHITESPACE)
         self.parent_url = parent_url
         self.recursion_level = recursion_level
         self.aggregate = aggregate
@@ -130,7 +137,7 @@ class UrlBase (object):
         if self.parent_url:
             assert not urlutil.url_needs_quoting(self.parent_url), \
                    "unquoted parent URL %r" % self.parent_url
-        url = absolute_url(base_url, base_ref, parent_url)
+        url = absolute_url(self.base_url, base_ref, parent_url)
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0] or "file"
 
@@ -303,7 +310,7 @@ class UrlBase (object):
         self.anchors = cache_data["anchors"]
         self.content_type = cache_data["content_type"]
         self.cached = True
-        if anchor_changed and self.valid and self.anchor:
+        if anchor_changed and self.valid:
             # recheck anchor
             if self.check_anchor():
                 # a new warning has been added - remove cached flag
@@ -388,7 +395,7 @@ class UrlBase (object):
         elif self.parent_url:
             # strip the parent url query and anchor
             urlparts = list(urlparse.urlsplit(self.parent_url))
-            urlparts[3] = urlparts[4] = ""
+            urlparts[4] = ""
             parent_url = urlparse.urlunsplit(urlparts)
             self.url = urljoin(parent_url, base_url, self.scheme)
         else:
@@ -426,8 +433,8 @@ class UrlBase (object):
             return
         # check for obfuscated IP address
         if iputil.is_obfuscated_ip(self.host):
-             ips = iputil.resolve_host(self.host)
-             if ips:
+            ips = iputil.resolve_host(self.host)
+            if ips:
                 self.add_warning(
                    _("URL %(url)s has obfuscated IP address %(ip)s") % \
                    {"url": self.base_url, "ip": ips.pop()},
@@ -455,7 +462,7 @@ class UrlBase (object):
         """Try to ask GeoIP database for country info."""
         if self.host:
             country = geoip.get_country(self.host)
-            if country is not None:
+            if country:
                 self.add_info(_("URL is located in %(country)s.") %
                 {"country": _(country)})
 
@@ -596,7 +603,7 @@ class UrlBase (object):
         return handler.follow
 
     def get_anchors (self):
-        """Store list of anchors for this URL. Precondition: this URL is
+        """Store anchors for this URL. Precondition: this URL is
         an HTML resource."""
         log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
         handler = linkparse.LinkFinder(self.add_anchor,
@@ -618,15 +625,17 @@ class UrlBase (object):
         self.anchors.append((url, line, column, name, base))
 
     def check_anchor (self):
-        """If URL was valid and has an anchor, check it. A warning is
-        logged and True is returned if the anchor is not found.
+        """If URL is valid, parseable and has an anchor, check it.
+        A warning is logged and True is returned if the anchor is not found.
         """
-        if not self.aggregate.config["anchors"]:
+        if not (self.anchor and self.aggregate.config["anchors"] and
+                self.valid and self.is_html()):
             return
         log.debug(LOG_CHECK, "checking anchor %r in %s", self.anchor, self.anchors)
         if any(x for x in self.anchors if x[0] == self.anchor):
             return
-        anchors = u",".join(u"`%s'" % x[0] for x in self.anchors)
+        anchornames = sorted(set(u"`%s'" % x[0] for x in self.anchors))
+        anchors = u", ".join(anchornames)
         args = {"name": self.anchor, "anchors": anchors}
         msg = u"%s %s" % (_("Anchor `%(name)s' not found.") % args,
                           _("Available anchors: %(anchors)s.") % args)
@@ -682,6 +691,8 @@ class UrlBase (object):
     def read_content (self):
         """Return data and data size for this URL.
         Can be overridden in subclasses."""
+        if self.size > self.MaxFilesizeBytes:
+            raise LinkCheckerError(_("File size too large"))
         data = self.url_connection.read()
         return data, len(data)
 
@@ -693,8 +704,7 @@ class UrlBase (object):
             self.set_title_from_content()
             if self.aggregate.config["anchors"]:
                 self.get_anchors()
-        if self.anchor:
-            self.check_anchor()
+        self.check_anchor()
         self.check_warningregex()
         # is it an intern URL?
         if not self.extern[0]:
@@ -712,17 +722,19 @@ class UrlBase (object):
                 self.scan_virus()
 
     def check_warningregex (self):
+        """Check if content matches a given regular expression."""
         warningregex = self.aggregate.config["warningregex"]
-        if warningregex:
-            log.debug(LOG_CHECK, "checking content")
-            try:
-                match = warningregex.search(self.get_content())
-                if match:
-                    self.add_warning(_("Found %(match)r in link contents.") %
-                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
-            except tuple(ExcList):
-                value = self.handle_exception()
-                self.set_result(unicode_safe(value), valid=False)
+        if not (warningregex and self.valid and self.is_parseable()):
+            return
+        log.debug(LOG_CHECK, "checking content")
+        try:
+            match = warningregex.search(self.get_content())
+            if match:
+                self.add_warning(_("Found %(match)r in link contents.") %
+                   {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
+        except tuple(ExcList):
+            value = self.handle_exception()
+            self.set_result(unicode_safe(value), valid=False)
 
     def check_size (self):
         """Check content size if it is zero or larger than a given
@@ -749,12 +761,11 @@ class UrlBase (object):
     def check_html (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the local HTML tidy module."""
-        try:
-            import tidy
-        except ImportError:
+        if not fileutil.has_module("tidy"):
             log.warn(LOG_CHECK, _("warning: tidy module is not available; " \
                  "download from http://utidylib.berlios.de/"))
             return
+        import tidy
         options = dict(output_html=0, show_warnings=1, quiet=True,
             input_encoding='utf8', output_encoding='utf8', tidy_mark=0)
         try:
@@ -923,22 +934,20 @@ class UrlBase (object):
     def parse_opera (self):
         """Parse an opera bookmark file."""
         log.debug(LOG_CHECK, "Parsing Opera bookmarks %s", self)
-        name = None
-        lineno = 0
-        for line in self.get_content().splitlines():
-            lineno += 1
-            line = line.strip()
-            if line.startswith("NAME="):
-                name = line[5:]
-            elif line.startswith("URL="):
-                url = line[4:]
-                if url and name is not None:
-                    url_data = get_url_from(url, self.recursion_level+1,
-                        self.aggregate, parent_url=self.url,
-                        line=lineno, name=name)
-                    self.aggregate.urlqueue.put(url_data)
-            else:
-                name = None
+        from ..bookmarks.opera import parse_bookmark_data
+        for url, name, lineno in parse_bookmark_data(self.get_content()):
+            url_data = get_url_from(url, self.recursion_level+1,
+               self.aggregate, parent_url=self.url, line=lineno, name=name)
+            self.aggregate.urlqueue.put(url_data)
+
+    def parse_chromium (self):
+        """Parse a Chromium or Google Chrome bookmark file."""
+        log.debug(LOG_CHECK, "Parsing Chromium bookmarks %s", self)
+        from ..bookmarks.chromium import parse_bookmark_data
+        for url, name in parse_bookmark_data(self.get_content()):
+            url_data = get_url_from(url, self.recursion_level+1,
+               self.aggregate, parent_url=self.url, name=name)
+            self.aggregate.urlqueue.put(url_data)
 
     def parse_text (self):
         """
@@ -1011,7 +1020,7 @@ class UrlBase (object):
     def get_temp_filename (self):
         """Get temporary filename for content to parse."""
         # store content in temporary file
-        fd, filename = fileutil.get_tempfile(suffix='.doc', prefix='lc_')
+        fd, filename = fileutil.get_temp_file(suffix='.doc', prefix='lc_')
         try:
             fd.write(self.get_content())
         finally:
@@ -1080,7 +1089,7 @@ class UrlBase (object):
           Indicates if URL data has been loaded from cache.
         - url_data.result: unicode
           Result string
-        - url_data.warnings: list of unicode
+        - url_data.warnings: list of tuples (tag, warning message)
           List of tagged warnings for this URL.
         - url_data.name: unicode string or None
           name of URL (eg. filename or link name)
@@ -1108,12 +1117,14 @@ class UrlBase (object):
           Cache key for this URL.
         - url_data.content_type: unicode
           MIME content type for URL content.
+        - url_data.level: int
+          Recursion level until reaching this URL from start URL
         """
         return dict(valid=self.valid,
           extern=self.extern[0],
           cached=self.cached,
           result=self.result,
-          warnings=[x[1] for x in self.warnings],
+          warnings=self.warnings[:],
           name=self.name or u"",
           title=self.get_title(),
           parent_url=self.parent_url or u"",
@@ -1129,9 +1140,12 @@ class UrlBase (object):
           column=self.column,
           cache_url_key=self.cache_url_key,
           content_type=self.get_content_type(),
+          level=self.recursion_level,
         )
 
     def to_wire (self):
+        """Return compact UrlData object with information from to_wire_dict().
+        """
         return CompactUrlData(self.to_wire_dict())
 
 
@@ -1162,9 +1176,11 @@ urlDataAttr = [
     'column',
     'cache_url_key',
     'content_type',
+    'level',
 ]
 
 class CompactUrlData (object):
+    """Store selected UrlData attributes in slots to minimize memory usage."""
     __slots__ = urlDataAttr
 
     def __init__(self, wired_url_data):
