@@ -22,9 +22,8 @@ from PyQt4 import QtCore, QtGui
 from .linkchecker_ui_main import Ui_MainWindow
 from .properties import set_properties, clear_properties
 from .statistics import set_statistics, clear_statistics
-from .progress import LinkCheckerProgress, StatusLogger
 from .debug import LinkCheckerDebug
-from .logger import GuiLogger, GuiLogHandler
+from .logger import GuiLogger, GuiLogHandler, StatusLogger
 from .help import HelpWindow
 from .options import LinkCheckerOptions
 from .checker import CheckerThread
@@ -34,8 +33,9 @@ from .updater import UpdateDialog
 from .urlmodel import UrlItemModel
 from .urlsave import urlsave
 from .settings import Settings
+from .recentdocs import RecentDocumentModel
 from .. import configuration, checker, director, add_intern_pattern, \
-    strformat, fileutil
+    strformat, fileutil, LinkCheckerError
 from ..containers import enum
 from .. import url as urlutil
 from ..checker import httpheaders
@@ -46,37 +46,82 @@ RegistryBase = "Bastian"
 Status = enum('idle', 'checking')
 
 
+def get_app_style ():
+    """Return appropriate QStyle object for the current platform to
+    be used in QApplication.setStyle().
+    Currently prefers Macintosh on OS X, else Plastique.
+    Style names are case insensitive.
+
+    See also
+    http://doc.trolltech.com/latest/gallery-macintosh.html
+    and
+    http://doc.trolltech.com/latest/gallery-plastique.html
+    """
+    if sys.platform == 'darwin':
+        style = "Macintosh"
+    else:
+        style = "Plastique"
+    return QtGui.QStyleFactory.create(style)
+
+
+def get_icon (name):
+    """Return QIcon with given pixmap resource name."""
+    icon = QtGui.QIcon()
+    icon.addPixmap(QtGui.QPixmap(name), QtGui.QIcon.Normal, QtGui.QIcon.Off)
+    return icon
+
+
 class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
+    """The main window displaying checked URLs."""
 
     log_url_signal = QtCore.pyqtSignal(object)
+    log_status_signal = QtCore.pyqtSignal(int, int, int, float)
     log_stats_signal = QtCore.pyqtSignal(object)
+    error_signal = QtCore.pyqtSignal(str)
 
     def __init__(self, parent=None, url=None):
         """Initialize UI."""
         super(LinkCheckerMain, self).__init__(parent)
         self.setupUi(self)
-        if url:
-            self.urlinput.setText(url)
         self.setWindowFlags(self.windowFlags() | QtCore.Qt.WindowContextHelpButtonHint)
         self.setWindowTitle(configuration.App)
         # app settings
         self.settings = Settings(RegistryBase, configuration.AppName)
         # init subdialogs
         self.options = LinkCheckerOptions(parent=self)
-        self.progress = LinkCheckerProgress(parent=self)
         self.debug = LinkCheckerDebug(parent=self)
-        self.checker = CheckerThread()
+        self.checker = CheckerThread(parent=self)
         self.contextmenu = ContextMenu(parent=self)
         self.editor = EditorWindow(parent=self)
         # Note: do not use QT assistant here because of the .exe packaging
         self.assistant = HelpWindow(self, self.get_qhcpath())
+        self.config_error = None
+        self.icon_start = get_icon(":/icons/start.png")
+        self.icon_stop = get_icon(":/icons/stop.png")
+        self.movie = QtGui.QMovie(":/icons/busy.gif")
+        self.movie.setCacheMode(QtGui.QMovie.CacheAll)
+        self.label_busy.setText(u"")
+        self.label_busy.setMovie(self.movie)
         # init the rest
+        self.init_url(url)
         self.init_treeview()
         self.connect_widgets()
         self.init_config()
+        self.read_config()
         self.init_app()
 
+    def init_url (self, url):
+        """Initialize URL input."""
+        documents = self.settings.read_recent_documents()
+        self.recent = RecentDocumentModel(parent=self, documents=documents)
+        self.urlinput.setModel(self.recent)
+        if url:
+            self.urlinput.setText(url)
+        elif documents:
+            self.urlinput.setText(documents[0])
+
     def init_app (self):
+        """Set window size and position, GUI options and reset status."""
         data = self.settings.read_geometry()
         if data["size"] is not None:
             self.resize(data["size"])
@@ -85,7 +130,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.options.set_options(self.settings.read_options())
         self.status = Status.idle
         self.actionSave.setEnabled(False)
-        self.set_statusbar(_("Ready."))
+        msg = self.config_error or _("Ready.")
+        self.set_statusmsg(msg)
 
     def get_qhcpath (self):
         """Helper function to search for the QHC help file in different
@@ -109,13 +155,20 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         Autoconnected methods have the form on_<objectname>_<signal>.
         """
         def set_idle ():
+            """Set application status to idle."""
             self.status = Status.idle
+            self.set_statusmsg(_("Check finished."))
+            self.controlButton.clicked.disconnect(self.checker.cancel)
         self.checker.finished.connect(set_idle)
         self.checker.terminated.connect(set_idle)
         self.log_url_signal.connect(self.model.log_url)
         self.log_stats_signal.connect(self.log_stats)
+        self.error_signal.connect(self.internal_error)
+        self.options.editor.saved.connect(self.read_config)
+        self.log_status_signal.connect(self.log_status)
 
     def init_treeview (self):
+        """Set treeview model and layout."""
         self.model = UrlItemModel()
         self.treeView.setModel(self.model)
         data = self.settings.read_treeviewcols()
@@ -126,6 +179,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         selectionModel.selectionChanged.connect(self.set_properties)
 
     def get_treeviewcols (self):
+        """Return URL treeview column widths."""
         return dict(
             col1=self.treeView.columnWidth(0),
             col2=self.treeView.columnWidth(1),
@@ -135,15 +189,23 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     def init_config (self):
         """Create a configuration object."""
         self.config = configuration.Configuration()
+        # set standard GUI configuration values
         self.config.logger_add("gui", GuiLogger)
         self.config["logger"] = self.config.logger_new('gui',
             signal=self.log_url_signal, stats=self.log_stats_signal)
         self.config["status"] = True
         self.config["status_wait_seconds"] = 2
         self.handler = GuiLogHandler(self.debug.log_msg_signal)
-        status = StatusLogger(self.progress.log_status_signal)
+        status = StatusLogger(self.log_status_signal)
         self.config.init_logging(status, handler=self.handler)
 
+    def read_config (self, filename=None):
+        """Read user and system configuration file."""
+        try:
+            self.config.read()
+        except LinkCheckerError, msg:
+            self.config_error = msg
+
     def set_config (self):
         """Set configuration."""
         data = self.options.get_options()
@@ -157,24 +219,43 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.config.reset_loglevel()
 
     def get_status (self):
+        """Return current application status."""
         return self._status
 
     def set_status (self, status):
+        """Set application status."""
         self._status = status
         if status == Status.idle:
-            self.progress.hide()
             self.aggregate = None
+            self.controlButton.setText(_("Start"))
+            self.controlButton.setIcon(self.icon_start)
             self.controlButton.setEnabled(True)
             self.actionSave.setEnabled(True)
             self.actionDebug.setEnabled(self.options.get_options()["debug"])
             self.treeView.sortByColumn(0, QtCore.Qt.AscendingOrder)
             self.treeView.setSortingEnabled(True)
+            self.treeView.scrollToTop()
+            self.movie.stop()
+            # Reset progress information.
+            self.label_active.setText(u"0")
+            self.label_queued.setText(u"0")
+            self.label_checked.setText(u"0")
+            self.label_busy.hide()
+            self.menubar.setEnabled(True)
+            self.urlinput.setEnabled(True)
         elif status == Status.checking:
             self.treeView.setSortingEnabled(False)
             self.debug.reset()
-            self.progress.reset()
-            self.progress.show()
-            self.controlButton.setEnabled(False)
+            self.set_statusmsg(u"Checking site...")
+            # disable commands
+            self.menubar.setEnabled(False)
+            self.urlinput.setEnabled(False)
+            # reset widgets
+            self.controlButton.setText(_("Stop"))
+            self.controlButton.setIcon(self.icon_stop)
+            self.controlButton.clicked.connect(self.checker.cancel)
+            self.movie.start()
+            self.label_busy.show()
 
     status = property(get_status, set_status)
 
@@ -199,6 +280,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.settings.save_geometry(dict(size=self.size(), pos=self.pos()))
         self.settings.save_treeviewcols(self.get_treeviewcols())
         self.settings.save_options(self.options.get_options())
+        self.settings.save_recent_documents(self.recent.get_documents())
         self.settings.sync()
         self.config.remove_loghandler(self.handler)
         if e is not None:
@@ -207,22 +289,34 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     @QtCore.pyqtSlot()
     def on_actionAbout_triggered (self):
         """Display about dialog."""
+        modules = u"<br>\n".join(configuration.get_modules_info())
         d = {
             "app": configuration.App,
             "appname": configuration.AppName,
             "copyright": configuration.HtmlCopyright,
+            "donateurl": configuration.DonateUrl,
+            "pyver": u"%d.%d.%d" % sys.version_info[:3],
+            "modules": modules,
+            "releasedate": configuration.ReleaseDate,
         }
         QtGui.QMessageBox.about(self, _(u"About %(appname)s") % d,
-            _(u"""<qt><p>%(appname)s checks HTML documents and websites
-for broken links.
+            _(u"""<qt><center>
+<h1>%(app)s</h1>
+<p>Released on %(releasedate)s
+<p>Python: %(pyver)s<br>
+%(modules)s
 <p>%(copyright)s
-<br>%(app)s is licensed under the
+<br>%(appname)s is licensed under the
 <a href="http://www.gnu.org/licenses/gpl.html">GPL</a>
 Version 2 or later.
-<p>Please consider a
-<a href="https://sourceforge.net/donate/index.php?group_id=1913">donation</a>
-to improve %(appname)s even more!
-</qt>""") % d)
+<p>If you like %(appname)s, consider one of several ways to
+<a href="%(donateurl)s">donate</a>. Thanks!
+</center></qt>""") % d)
+
+    @QtCore.pyqtSlot()
+    def on_actionDonate_triggered (self):
+        """Open donation URL."""
+        webbrowser.open(configuration.DonateUrl)
 
     @QtCore.pyqtSlot()
     def on_actionDebug_triggered (self):
@@ -246,18 +340,33 @@ to improve %(appname)s even more!
         if self.status == Status.idle:
             self.check()
 
-    on_controlButton_clicked = on_urlinput_returnPressed = start
+    on_urlinput_returnPressed = start
+
+    def cancel (self):
+        """Note that checking is canceled."""
+        self.controlButton.setEnabled(False)
+        self.set_statusmsg(_(u"Closing pending connections..."))
+
+    @QtCore.pyqtSlot()
+    def on_controlButton_clicked (self):
+        """Start or Cancel has been clicked."""
+        if self.status == Status.idle:
+            self.start()
+        elif self.status == Status.checking:
+            self.cancel()
+        else:
+            raise ValueError("Invalid application status %r" % self.status)
 
     def get_url (self):
         """Return URL to check from the urlinput widget."""
-        url = unicode(self.urlinput.text()).strip()
+        url = strformat.stripall(unicode(self.urlinput.text()))
         if url.startswith(u"www."):
             url = u"http://%s" % url
         elif url.startswith(u"ftp."):
             url = u"ftp://%s" % url
         elif url and u":" not in url:
-            # Look for local filename, else assume it's an HTTP URL.
-            if not os.path.isfile(url):
+            # Look for local file, else assume it's an HTTP URL.
+            if not os.path.exists(url):
                 url = u"http://%s" % url
         return url
 
@@ -270,28 +379,37 @@ to improve %(appname)s even more!
         aggregate = director.get_aggregate(self.config)
         url = self.get_url()
         if not url:
-            self.set_statusbar(_("Error, empty URL"))
+            self.set_statusmsg(_("Error, empty URL"))
             return
-        self.set_statusbar(_("Checking '%s'.") % strformat.limit(url, 40))
+        self.set_statusmsg(_("Checking '%s'.") % strformat.limit(url, 40))
         url_data = checker.get_url_from(url, 0, aggregate)
         try:
             add_intern_pattern(url_data, self.config)
         except UnicodeError:
-            self.set_statusbar(_("Error, invalid URL `%s'.") %
+            self.set_statusmsg(_("Error, invalid URL `%s'.") %
                                   strformat.limit(url, 40))
             return
+        self.recent.add_document(url)
         aggregate.urlqueue.put(url_data)
         self.aggregate = aggregate
         # check in background
-        self.checker.check(self.aggregate, self.progress)
+        self.checker.check(self.aggregate)
         self.status = Status.checking
 
     def set_properties (self, selected, deselected):
         """Set URL properties for selected item."""
-        index = selected.indexes()[0]
-        urlitem = self.model.getUrlItem(index)
-        if urlitem is not None:
-            set_properties(self, urlitem.url_data)
+        indexes = selected.indexes()
+        if len(indexes):
+            index = indexes[0]
+            urlitem = self.model.getUrlItem(index)
+            if urlitem is not None:
+                set_properties(self, urlitem.url_data)
+        selected_rows = len(self.treeView.selectionModel().selectedRows())
+        if selected_rows:
+            self.set_statusmsg(_n("%d URL selected.", "%d URLs selected",
+                               selected_rows) % selected_rows)
+        else:
+            self.set_statusmsg(_("Ready."))
 
     def on_treeView_customContextMenuRequested (self, point):
         """Show item context menu."""
@@ -300,13 +418,6 @@ to improve %(appname)s even more!
             self.contextmenu.enableFromItem(urlitem)
             self.contextmenu.popup(QtGui.QCursor.pos())
 
-    @QtCore.pyqtSlot()
-    def on_actionViewProperties_triggered (self):
-        """View URL data properties in a separate window."""
-        urlitem = self.model.getUrlItem(self.treeView.currentIndex())
-        if urlitem is not None:
-            self.view_item_properties(urlitem)
-
     @QtCore.pyqtSlot()
     def on_actionViewOnline_triggered (self):
         """View item URL online."""
@@ -330,7 +441,9 @@ to improve %(appname)s even more!
                              urlitem.url_data.line, urlitem.url_data.column)
 
     def view_source (self, url, line, col):
+        """View URL source in editor window."""
         self.editor.setWindowTitle(u"View %s" % url)
+        self.editor.setUrl(url)
         info, data = urlutil.get_content(url, proxy=self.config["proxy"])
         if (info, data) == (None, None):
             self.editor.setText(u"An error occurred retreiving URL `%s'." % url)
@@ -354,9 +467,23 @@ to improve %(appname)s even more!
             event = QtCore.QEvent(QtCore.QEvent.Clipboard)
             QtGui.QApplication.sendEvent(clipboard, event)
 
-    def set_statusbar (self, msg):
-        """Show status message in status bar."""
-        self.statusBar.showMessage(msg)
+    def set_statusmsg (self, msg):
+        """Show given status message."""
+        if len(msg) > 30:
+            self.label_status.setToolTip(msg)
+            msg = msg[:27]+u"..."
+        self.label_status.setText(msg)
+
+    def log_status (self, checked, in_progress, queued, duration):
+        """Update number of checked, active and queued URLs."""
+        self.label_checked.setText(u"%d" % checked)
+        self.label_active.setText(u"%d" % in_progress)
+        self.label_queued.setText(u"%d" % queued)
 
     def log_stats (self, statistics):
+        """Set statistic information for selected URL."""
         set_statistics(self, statistics)
+
+    def internal_error (self, msg):
+        """Display internal error message. Triggered by sys.excepthook()."""
+        QtGui.QMessageBox.warning(self, _(u"LinkChecker internal error"), msg)
