@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ Fields = dict(
     dlsize=_("Size"),
     checktime=_("Check time"),
     url=_("URL"),
+    level=_("Level"),
 )
 del _
 
@@ -54,6 +55,7 @@ ContentTypes = dict(
     other=0,
 )
 
+
 class LogStatistics (object):
     """Gather log statistics:
     - number of errors, warnings and valid links
@@ -63,9 +65,11 @@ class LogStatistics (object):
     """
 
     def __init__ (self):
+        """Initialize log statistics."""
         self.reset()
 
     def reset (self):
+        """Reset all log statistics to default values."""
         # number of logged urls
         self.number = 0
         # number of encountered errors
@@ -84,6 +88,7 @@ class LogStatistics (object):
         self.avg_number = 0
 
     def log_url (self, url_data, do_print):
+        """Log URL statistics."""
         self.number += 1
         if not url_data.valid:
             self.errors += 1
@@ -121,7 +126,7 @@ class Logger (object):
     Base class for logging of checked urls. It defines the public API
     (see below) and offers basic functionality for all loggers.
 
-    Each logger has to offer the following functions:
+    Each logger offers the following functions:
 
     * start_output()
         Initialize and start log output. Most loggers print a comment
@@ -133,6 +138,15 @@ class Logger (object):
         Log a checked URL. The url_data object is a transport form of
         the UrlData class. The do_print flag indicates if this URL
         should be logged or just used to update internal statistics.
+
+    Each subclassed logger must implement the following functions:
+
+    * start_output()
+        Also call the base class implementation of this.
+    * end_output()
+        See above.
+    * log_url(url_data)
+        Log a checked URL. Called by log_filter_url if do_print is True.
     """
 
     def __init__ (self, **args):
@@ -187,9 +201,10 @@ class Logger (object):
         elif 'fd' in args:
             self.fd = args['fd']
         else:
-            self.fd = codecs.getwriter(self.output_encoding)(sys.stdout)
+            self.fd = self.create_fd()
 
     def start_fileoutput (self):
+        """Start output to configured file."""
         path = os.path.dirname(self.filename)
         try:
             if path and not os.path.isdir(path):
@@ -206,6 +221,9 @@ class Logger (object):
 
     def create_fd (self):
         """Create open file descriptor."""
+        if self.filename is None:
+            return i18n.get_encoded_writer(encoding=self.output_encoding,
+                                           errors=self.codec_errors)
         return codecs.open(self.filename, "wb", self.output_encoding,
                            self.codec_errors)
 
