@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,17 +11,16 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 A dummy logger.
 """
-
-import linkcheck.logger
+from . import Logger
 
 
-class NoneLogger (linkcheck.logger.Logger):
+class NoneLogger (Logger):
     """
     Dummy logger printing nothing.
     """
@@ -38,13 +37,13 @@ class NoneLogger (linkcheck.logger.Logger):
         """
         pass
 
-    def new_url (self, url_data):
+    def log_filter_url (self, url_data, do_filter):
         """
         Do nothing.
         """
         pass
 
-    def end_output (self, linknumber=-1):
+    def end_output (self):
         """
         Do nothing.
         """
