@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,9 +31,10 @@ import math
 import time
 import urlparse
 import pydoc
+from . import i18n
 
 
-def unicode_safe (s, encoding="iso-8859-1"):
+def unicode_safe (s, encoding=i18n.default_encoding):
     """Get unicode string without raising encoding errors. Unknown
     characters of the given encoding will be ignored.
 
